@@ -32,6 +32,7 @@ from notebook.utils import (
     to_api_path,
 )
 from notebook.base.handlers import AuthenticatedFileHandler
+from notebook.transutils import _
 
 try:
     from os.path import samefile
@@ -244,6 +245,14 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """Build the common base of a contents model"""
         os_path = self._get_os_path(path)
         info = os.lstat(os_path)
+        
+        try:
+            # size of file 
+            size = info.st_size
+        except (ValueError, OSError):
+            self.log.warning('Unable to get size.')
+            size = None
+        
         try:
             last_modified = tz.utcfromtimestamp(info.st_mtime)
         except (ValueError, OSError):
@@ -269,6 +278,8 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         model['content'] = None
         model['format'] = None
         model['mimetype'] = None
+        model['size'] = size
+
         try:
             model['writable'] = os.access(os_path, os.W_OK)
         except OSError:
@@ -295,6 +306,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
         model = self._base_model(path)
         model['type'] = 'directory'
+        model['size'] = None
         if content:
             model['content'] = contents = []
             os_dir = self._get_os_path(path)
@@ -332,6 +344,7 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
 
         return model
 
+
     def _file_model(self, path, content=True, format=None):
         """Build a model for a file
 
@@ -372,13 +385,15 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         """
         model = self._base_model(path)
         model['type'] = 'notebook'
+        os_path = self._get_os_path(path)
+        
         if content:
-            os_path = self._get_os_path(path)
             nb = self._read_notebook(os_path, as_version=4)
             self.mark_trusted_cells(nb, path)
             model['content'] = nb
             model['format'] = 'json'
             self.validate_notebook_model(model)
+            
         return model
 
     def get(self, path, content=True, type=None, format=None):
@@ -491,13 +506,28 @@ class FileContentsManager(FileManagerMixin, ContentsManager):
         if not os.path.exists(os_path):
             raise web.HTTPError(404, u'File or directory does not exist: %s' % os_path)
 
+        def _check_trash(os_path):
+            if sys.platform in {'win32', 'darwin'}:
+                return True
+
+            # It's a bit more nuanced than this, but until we can better
+            # distinguish errors from send2trash, assume that we can only trash
+            # files on the same partition as the home directory.
+            file_dev = os.stat(os_path).st_dev
+            home_dev = os.stat(os.path.expanduser('~')).st_dev
+            return file_dev == home_dev
+
         if self.delete_to_trash:
-            self.log.debug("Sending %s to trash", os_path)
-            # Looking at the code in send2trash, I don't think the errors it
-            # raises let us distinguish permission errors from other errors in
-            # code. So for now, just let them all get logged as server errors.
-            send2trash(os_path)
-            return
+            if _check_trash(os_path):
+                self.log.debug("Sending %s to trash", os_path)
+                # Looking at the code in send2trash, I don't think the errors it
+                # raises let us distinguish permission errors from other errors in
+                # code. So for now, just let them all get logged as server errors.
+                send2trash(os_path)
+                return
+            else:
+                self.log.warning("Skipping trash for %s, on different device "
+                                 "to home directory", os_path)
 
         if os.path.isdir(os_path):
             listing = os.listdir(os_path)
