@@ -4,9 +4,12 @@
 define([
     'jquery',
     'base/js/utils',
+    'base/js/i18n',
     'base/js/dialog',
-], function($, utils, dialog) {
+    'underscore'
+], function($, utils, i18n, dialog, _) {
     "use strict";
+
     var platform = utils.platform;
 
     var QuickHelp = function (options) {
@@ -33,42 +36,51 @@ define([
         // Mac OS X specific
         cmd_ctrl = 'Cmd-';
         platform_specific = [
-            { shortcut: "Cmd-Up",     help:"go to cell start"  },
-            { shortcut: "Cmd-Down",   help:"go to cell end"  },
-            { shortcut: "Alt-Left",   help:"go one word left"  },
-            { shortcut: "Alt-Right",  help:"go one word right"  },
-            { shortcut: "Alt-Backspace",      help:"delete word before"  },
-            { shortcut: "Alt-Delete",         help:"delete word after"  },
+            { shortcut: "Cmd-Up", help:i18n.msg._("go to cell start") },
+            { shortcut: "Cmd-Down", help:i18n.msg._("go to cell end") },
+            { shortcut: "Alt-Left", help:i18n.msg._("go one word left") },
+            { shortcut: "Alt-Right", help:i18n.msg._("go one word right") },
+            { shortcut: "Alt-Backspace", help:i18n.msg._("delete word before") },
+            { shortcut: "Alt-Delete", help:i18n.msg._("delete word after") },
+            { shortcut: "Cmd-Shift-z", help:i18n.msg._("redo") },
+            { shortcut: "Cmd-Shift-u", help:i18n.msg._("redo selection") },
+            { shortcut: "Ctrl-k", help:i18n.msg._("emacs-style line kill") },
+            { shortcut: "Cmd-Backspace", help:i18n.msg._("delete line left of cursor") },
+            { shortcut: "Cmd-Delete", help:i18n.msg._("delete line right of cursor") }
         ];
     } else {
         // PC specific
         platform_specific = [
-            { shortcut: "Ctrl-Home",  help:"go to cell start"  },
-            { shortcut: "Ctrl-Up",     help:"go to cell start"  },
-            { shortcut: "Ctrl-End",   help:"go to cell end"  },
-            { shortcut: "Ctrl-Down",  help:"go to cell end"  },
-            { shortcut: "Ctrl-Left",  help:"go one word left"  },
-            { shortcut: "Ctrl-Right", help:"go one word right"  },
-            { shortcut: "Ctrl-Backspace", help:"delete word before"  },
-            { shortcut: "Ctrl-Delete",    help:"delete word after"  },
+            { shortcut: "Ctrl-Home", help:i18n.msg._("go to cell start") },
+            { shortcut: "Ctrl-Up", help:i18n.msg._("go to cell start") },
+            { shortcut: "Ctrl-End", help:i18n.msg._("go to cell end") },
+            { shortcut: "Ctrl-Down", help:i18n.msg._("go to cell end") },
+            { shortcut: "Ctrl-Left", help:i18n.msg._("go one word left") },
+            { shortcut: "Ctrl-Right", help:i18n.msg._("go one word right") },
+            { shortcut: "Ctrl-Backspace", help:i18n.msg._("delete word before")},
+            { shortcut: "Ctrl-Delete", help:i18n.msg._("delete word after")},
+            { shortcut: "Ctrl-y", help:i18n.msg._("redo")},
+            { shortcut: "Alt-u", help:i18n.msg._("redo selection") }
         ];
     }
 
     var cm_shortcuts = [
-        { shortcut:"Tab",   help:"code completion or indent" },
-        { shortcut:"Shift-Tab",   help:"tooltip" },
-        { shortcut: cmd_ctrl + "]",   help:"indent"  },
-        { shortcut: cmd_ctrl + "[",   help:"dedent"  },
-        { shortcut: cmd_ctrl + "a",   help:"select all"  },
-        { shortcut: cmd_ctrl + "z",   help:"undo"  },
-        { shortcut: cmd_ctrl + "Shift-z",   help:"redo"  },
-        { shortcut: cmd_ctrl + "y",   help:"redo"  },
+        { shortcut:"Tab", help:i18n.msg._("code completion or indent") },
+        { shortcut:"Shift-Tab", help:i18n.msg._("tooltip") },
+        { shortcut: cmd_ctrl + "]", help:i18n.msg._("indent") },
+        { shortcut: cmd_ctrl + "[", help:i18n.msg._("dedent") },
+        { shortcut: cmd_ctrl + "a", help:i18n.msg._("select all") },
+        { shortcut: cmd_ctrl + "z", help:i18n.msg._("undo") },
+        { shortcut: cmd_ctrl + "/", help:i18n.msg._("comment") },
+        { shortcut: cmd_ctrl + "d", help:i18n.msg._("delete whole line") },
+        { shortcut: cmd_ctrl + "u", help:i18n.msg._("undo selection") },
+        { shortcut: "Insert", help:i18n.msg._("toggle overwrite flag") }
     ].concat( platform_specific );
     
     var mac_humanize_map = {
         // all these are unicode, will probably display badly on anything except macs.
         // these are the standard symbol that are used in MacOS native menus
-        // cf http://apple.stackexchange.com/questions/55727/
+        // cf https://apple.stackexchange.com/questions/55727/
         // for htmlentities and/or unicode value
         'cmd':'⌘',
         'shift':'⇧',
@@ -96,24 +108,24 @@ define([
     };
 
     var default_humanize_map = {
-        'shift':'Shift',
-        'alt':'Alt',
-        'up':'Up',
-        'down':'Down',
-        'left':'Left',
-        'right':'Right',
-        'tab':'Tab',
-        'capslock':'Caps Lock',
-        'esc':'Esc',
-        'ctrl':'Ctrl',
-        'enter':'Enter',
-        'pageup':'Page Up',
-        'pagedown':'Page Down',
-        'home':'Home',
-        'end':'End',
-        'space':'Space',
-        'backspace':'Backspace',
-        '-':'Minus'
+        'shift':i18n.msg._('Shift'),
+        'alt':i18n.msg._('Alt'),
+        'up':i18n.msg._('Up'),
+        'down':i18n.msg._('Down'),
+        'left':i18n.msg._('Left'),
+        'right':i18n.msg._('Right'),
+        'tab':i18n.msg._('Tab'),
+        'capslock':i18n.msg._('Caps Lock'),
+        'esc':i18n.msg._('Esc'),
+        'ctrl':i18n.msg._('Ctrl'),
+        'enter':i18n.msg._('Enter'),
+        'pageup':i18n.msg._('Page Up'),
+        'pagedown':i18n.msg._('Page Down'),
+        'home':i18n.msg._('Home'),
+        'end':i18n.msg._('End'),
+        'space':i18n.msg._('Space'),
+        'backspace':i18n.msg._('Backspace'),
+        '-':i18n.msg._('Minus')
         };
     
     var humanize_map;
@@ -124,7 +136,7 @@ define([
         humanize_map = default_humanize_map;
     }
 
-    var special_case = { pageup: "PageUp", pagedown: "Page Down" };
+    var special_case = { pageup: i18n.msg._("PageUp"), pagedown: i18n.msg._("Page Down") };
     
     function humanize_key(key){
         if (key.length === 1){
@@ -153,13 +165,22 @@ define([
         return hum;
     }
 
-    function humanize_shortcut(shortcut){
+    function _humanize_sequence(sequence){
+        var joinchar = ',';
+        var hum = _.map(sequence.replace(/meta/g, 'cmd').split(','), _humanize_shortcut).join(joinchar);
+        return hum;
+    }
+
+    function _humanize_shortcut(shortcut){
         var joinchar = '-';
         if (platform === 'MacOS'){
             joinchar = '';
         }
-        var sh = _.map(shortcut.split('-'), humanize_key ).join(joinchar);
-        return '<kbd>'+sh+'</kbd>';
+        return _.map(shortcut.split('-'), humanize_key ).join(joinchar);
+    }
+
+    function humanize_shortcut(shortcut){
+        return '<kbd>'+_humanize_shortcut(shortcut)+'</kbd>';
     }
     
 
@@ -178,19 +199,15 @@ define([
             $(this.shortcut_dialog).modal("toggle");
             return;
         }
-        var command_shortcuts = this.keyboard_manager.command_shortcuts.help();
-        var edit_shortcuts = this.keyboard_manager.edit_shortcuts.help();
-        var help, shortcut;
-        var i, half, n;
         var element = $('<div/>');
 
         // The documentation
         var doc = $('<div/>').addClass('alert alert-info');
-        doc.append(
-            'The Jupyter Notebook has two different keyboard input modes. <b>Edit mode</b> '+
-            'allows you to type code/text into a cell and is indicated by a green cell '+
-            'border. <b>Command mode</b> binds the keyboard to notebook level actions '+
-            'and is indicated by a grey cell border with a blue left margin.'
+        doc.append(i18n.msg._('The Jupyter Notebook has two different keyboard input modes.'))
+           .append(' ')
+           .append(i18n.msg._('<b>Edit mode</b> allows you to type code or text into a cell and is indicated by a green cell border.'))
+           .append(' ')
+           .append(i18n.msg._('<b>Command mode</b> binds the keyboard to notebook level commands and is indicated by a grey cell border with a blue left margin.')
         );
         element.append(doc);
         if (platform === 'MacOS') {
@@ -208,8 +225,13 @@ define([
         var edit_div = this.build_edit_help(cm_shortcuts);
         element.append(edit_div);
 
+        // This statement is used simply so that message extraction
+        // will pick up the strings.  The actual setting of the text
+        // for the button is in dialog.js.
+        var button_labels = [ i18n.msg._("Close") ];
+
         this.shortcut_dialog = dialog.modal({
-            title : "Keyboard shortcuts",
+            title : i18n.msg._("Keyboard shortcuts"),
             body : element,
             destroy : false,
             buttons : {
@@ -224,13 +246,13 @@ define([
     };
 
     QuickHelp.prototype.build_key_names = function () {
-       var key_names_mac =  [{ shortcut:"⌘", help:"Command" },
-                    { shortcut:"⌃", help:"Control" },
-                    { shortcut:"⌥", help:"Option" },
-                    { shortcut:"⇧", help:"Shift" },
-                    { shortcut:"↩", help:"Return" },
-                    { shortcut:"␣", help:"Space" },
-                    { shortcut:"⇥", help:"Tab" }];
+       var key_names_mac =  [{ shortcut:"⌘", help:i18n.msg._("Command") },
+                    { shortcut:"⌃", help:i18n.msg._("Control") },
+                    { shortcut:"⌥", help:i18n.msg._("Option") },
+                    { shortcut:"⇧", help:i18n.msg._("Shift") },
+                    { shortcut:"↩", help:i18n.msg._("Return") },
+                    { shortcut:"␣", help:i18n.msg._("Space") },
+                    { shortcut:"⇥", help:i18n.msg._("Tab") }];
         var i, half, n;
         var div = $('<div/>').append('Mac OS X modifier keys:');
         var sub_div = $('<div/>').addClass('container-fluid');
@@ -251,15 +273,35 @@ define([
 
 
     QuickHelp.prototype.build_command_help = function () {
+        var that = this;
         var command_shortcuts = this.keyboard_manager.command_shortcuts.help();
-        return build_div('<h4>Command Mode (press <kbd>Esc</kbd> to enable)</h4>', command_shortcuts);
+        var cmdkey = '<kbd>'+i18n.msg._('Esc')+'</kbd>';
+        var div = build_div('<h4>'+i18n.msg.sprintf(i18n.msg._('Command Mode (press %s to enable)'),cmdkey)+'</h4>', command_shortcuts);
+        var edit_button = $('<button/>')
+            .text(i18n.msg._("Edit Shortcuts"))
+            .addClass('btn btn-xs btn-default pull-right')
+            .attr('href', '#')
+            .attr('title', i18n.msg._('edit command-mode keyboard shortcuts'))
+            .click(function () {
+                // close this dialog
+                $(that.shortcut_dialog).modal("toggle");
+                // and open the next one
+                $(that.shortcut_dialog).on('hidden.bs.modal', function (e) {
+                    that.keyboard_manager.actions.call(
+                        'jupyter-notebook:edit-command-mode-keyboard-shortcuts'
+                    );
+                });
+            });
+        div.find('h4').append(edit_button);
+        return div;
     };
 
     
     QuickHelp.prototype.build_edit_help = function (cm_shortcuts) {
         var edit_shortcuts = this.keyboard_manager.edit_shortcuts.help();
-        edit_shortcuts = jQuery.merge(jQuery.merge([], cm_shortcuts), edit_shortcuts);
-        return build_div('<h4>Edit Mode (press <kbd>Enter</kbd> to enable)</h4>', edit_shortcuts);
+        var enterkey = '<kbd>'+i18n.msg._('Enter')+'</kbd>';
+        edit_shortcuts = $.merge($.merge([], cm_shortcuts), edit_shortcuts);
+        return build_div('<h4>'+i18n.msg.sprintf(i18n.msg._('Edit Mode (press %s to enable)'),enterkey)+'</h4>', edit_shortcuts);
     };
 
     var build_one = function (s) {
@@ -301,6 +343,7 @@ define([
 
     return {'QuickHelp': QuickHelp,
       humanize_shortcut: humanize_shortcut,
-      humanize_sequence: humanize_sequence
+      humanize_sequence: humanize_sequence,
+      _humanize_sequence: _humanize_sequence,
   };
 });
